randomize();

enum SECTOR_TYPES
{
	asteroid_field,
	squid_fog,
	minefield,
	scarab_patrol,
	miniboss,
	
}

event_inherited();

room_width_third = room_width / 3;
room_height_third = room_height / 3;

view_number = 0;

sector_x = 0;
sector_y = 0;

sectors = ds_map_create();

function sector_distance_from_center()
{
	return point_distance(0, 0, sector_x, sector_y);	
}

function sector_get_code(target_sector_x, target_sector_y)
{
	return string(target_sector_x) + "," + string(target_sector_y);
}

function sector_generate(target_sector_x, target_sector_y, asteroid_clusters, scarab_clusters, mine_clusters, fog_clusters, squid_clusters, loot)
{
	
	var x1 = room_width_third * (target_sector_x - sector_x + 1);
	var y1 = room_height_third * (target_sector_y - sector_y + 1);
	var x2 = x1 + room_width_third;
	var y2 = y1 + room_height_third;
	
	repeat(asteroid_clusters)
	{
		var start_x = irandom_range(x1 + 80, x2 - 80);
		var start_y = irandom_range(y1 + 80, y2 - 80);

		repeat(irandom(12))
		{
			var xx = start_x + irandom_range(-80, 80);
			var yy = start_y + irandom_range(-80, 80);
			instance_create_layer(xx, yy, "InstancesBelow", choose(obj_rock, obj_rock, obj_rock, obj_rock, obj_rock, obj_rock, obj_rock, obj_rock_large));
		}
	}
	
	repeat(scarab_clusters)
	{
		var start_x = irandom_range(x1 + 128, x2 - 128);
		var start_y = irandom_range(y1 + 128, y2 - 128);

		repeat(irandom(6))
		{
			var xx = start_x + irandom_range(-128, 128);
			var yy = start_y + irandom_range(-128, 128);
			instance_create_layer(xx, yy, "Instances", choose(obj_scarab));
		}
	}
	
	repeat(mine_clusters)
	{
		var start_x = irandom_range(x1 + 128, x2 - 128);
		var start_y = irandom_range(y1 + 128, y2 - 128);

		repeat(irandom(10))
		{
			var xx = start_x + irandom_range(-128, 128);
			var yy = start_y + irandom_range(-128, 128);
			instance_create_layer(xx, yy, "Instances", choose(obj_mine));
		}
	}
	
	repeat(fog_clusters)
	{
		var start_x = irandom_range(x1 + 80, x2 - 80);
		var start_y = irandom_range(y1 + 80, y2 - 80);

		repeat(irandom(15))
		{
			var xx = start_x + irandom_range(-80, 80);
			var yy = start_y + irandom_range(-80, 80);
			instance_create_layer(xx, yy, "InstancesTop", choose(obj_mystery_fog));
		}
	}
	
	repeat(squid_clusters)
	{
		var start_x = irandom_range(x1 + 100, x2 - 100);
		var start_y = irandom_range(y1 + 100, y2 - 100);

		repeat(irandom(8))
		{
			var xx = start_x + irandom_range(-100, 100);
			var yy = start_y + irandom_range(-100, 100);
			instance_create_layer(xx, yy, "Instances", choose(obj_nokkasquid));
		}
	}
	
	repeat(loot)
	{
		var start_x = irandom_range(x1 + 100, x2 - 100);
		var start_y = irandom_range(y1 + 100, y2 - 100);


		pickup1 = instance_create_layer(start_x, start_y,"Instances", obj_pickup);
		pickup1.pickup_type = choose(pickup.healthpack, pickup.healthpack, pickup.healthpack, pickup.ammo_laser,pickup.ammo_laser, pickup.ammo_laser, pickup.ammo_missile, pickup.ammo_missile, pickup.ammo_star_gun, pickup.ammo_laser, pickup.ammo_missile, pickup.ammo_star_gun, pickup.healthpack);
	}
}


function sector_load(target_sector_x, target_sector_y)
{
	var sector_code = sector_get_code(target_sector_x, target_sector_y);
	var x1 = room_width_third * (target_sector_x - sector_x + 1);
	var y1 = room_height_third * (target_sector_y - sector_y + 1);
	var x2 = x1 + room_width_third;
	var y2 = y1 + room_height_third;
	
	if (ds_map_exists(sectors, sector_code))
	{
		// Load sector data if it exists
		var sector_data = sectors[? sector_code];
		
		for(var i = 0; i < array_length(sector_data); i++)
		{
			var e_data = sector_data[i];
			e_data.instantiate(x1, y1);
		}
		show_debug_message("Loading sector " + string(target_sector_x) + "," + string(target_sector_y));
	}
	else
	{
		// Generate new sector if no previous data is found
		var sector_dist = sector_distance_from_center();
	
		var type = choose(SECTOR_TYPES.asteroid_field, SECTOR_TYPES.minefield, SECTOR_TYPES.scarab_patrol, SECTOR_TYPES.squid_fog);
		
		with(obj_fragment)
		{
			// Special sector types for fragments
			if ( x > x1 && x < x2 && y > y1 && y < y2 && !follow_player && type != -1)
			{
				type = sector_type;
				type = -1;
			}
		}
		var asteroids = 0;
		var mines = 0;
		var scarabs = 0;
		var fog = 0;
		var squids = 0;
		var loot = 0;
		
		if (sector_dist > 1)
		{
			loot = choose(0, 0, 1, 1, 1, 1, 1, 2, 2, 2);	
		}
		if (sector_dist > 9)
		{
			loot = choose(0, 0, 1, 2, 3, 4);
		}
		switch(type)
		{
			case SECTOR_TYPES.asteroid_field:
			{
				asteroids = 15 + irandom( 2 * sector_dist);
				
				if (irandom(max( 3, 8 - sector_dist) ) == 1)
				{
					mines = irandom(5);
				}
				
				if (irandom(max( 3, 12 - sector_dist) ) == 1)
				{
					scarabs = irandom(3);
				}
				
				if (irandom(max( 4, 15 - sector_dist) ) == 1)
				{
					fog = irandom(5);
				}
				
				if (irandom(max( 4, 20 - sector_dist) ) == 1)
				{
					squids = irandom(3);
				}			
				
				break;
			}
			
			case SECTOR_TYPES.minefield:
			{
				mines = 10 + irandom( 2 * sector_dist);
				asteroids = irandom(8);
				
				if (irandom(max( 3, 12 - sector_dist) ) == 1)
				{
					scarabs = irandom(3);
				}
				
				if (irandom(max( 4, 15 - sector_dist) ) == 1)
				{
					fog = irandom(5);
				}
				
				if (irandom(max( 4, 20 - sector_dist) ) == 1)
				{
					squids = irandom(3);
				}			
				
				break;
			}
			
			case SECTOR_TYPES.scarab_patrol:
			{
				scarabs = 5 + irandom(sector_dist);
				mines = irandom(5);
				asteroids = irandom(8);
				
				if (irandom(max( 4, 15 - sector_dist) ) == 1)
				{
					fog = irandom(5);
				}
				
				if (irandom(max( 6, 20 - sector_dist) ) == 1)
				{
					squids = irandom(3);
				}			
				
				break;
			}
			
			case SECTOR_TYPES.squid_fog:
			{
				squids = 5 + irandom(sector_dist);
				fog = 12 + irandom(5);
				asteroids = irandom(8);					
				
				break;
			}
			
			case SECTOR_TYPES.miniboss:
			{
				asteroids = 5;
				
				if (irandom(4) == 1)
				{
					mines = irandom_range(3, 4);
				}
				else if (irandom(3) == 1)
				{
					scarabs = irandom_range(3, 4);
				}
				else if (irandom(2) == 1)
				{
					squids = irandom_range(3, 4);
				}
				else
				{
					asteroids = irandom_range(20, 25);;
				}
				
				break;
			}
			
		}
		
		sector_generate(target_sector_x, target_sector_y, asteroids, mines, scarabs, fog, squids, loot);
		
	}
	
	
}

function sector_unload(target_sector_x, target_sector_y)
{
	var x1 = room_width_third * (target_sector_x - sector_x + 1);
	var y1 = room_height_third * (target_sector_y - sector_y + 1);
	var x2 = x1 + room_width_third;
	var y2 = y1 + room_height_third;
	
	var sector_data = array_create(0);
	
	with(obj_entity)
	{
		// Check if entity is within the sector bounds
		if (x >= x1 && x < x2 && y >= y1 && y < y2)
		{
			if (!always_active)
			{
				if (is_persistent)
				{
					// Only save persistent stuff
					var e_data = new entity_data(id, x1, y1);
					array_push(sector_data, e_data);
				}
		
				// Destroy instance anyway
				instance_destroy();
			}
		}
	}
	
	var sector_code = sector_get_code(target_sector_x, target_sector_y);
	sectors[? sector_code] = sector_data;
}


sector_generate(-1, -1, 10, 0, 0, irandom(3), 0, 0);
sector_generate(0, -1, 10, 0, 0, irandom(3), 0, 0);
sector_generate(1, -1, 10, 0, 0, irandom(3), 0, 0);

sector_generate(-1, 0, 10, 0, 0, irandom(3), 0, 0);
sector_generate(1, 0, 10, 0, 0, irandom(3), 0, 0);

sector_generate(-1, 1, 10, 0, 0, irandom(3), 0, 0);
sector_generate(0, 1, 10, 0, 0, irandom(3), 0, 0);
sector_generate(1, 1, 10, 0, 0, irandom(3), 0, 0);


instance_create_layer(500 + room_width_third * 5, 500, "InstancesBottom", obj_fragment);

instance_create_layer(-1.5 * room_width_third + 500, -3 * room_height_third + 500, "InstancesBottom", obj_fragment);

instance_create_layer(-6 * room_width_third + 500, 2 * room_height_third + 500, "InstancesBottom", obj_fragment);

instance_create_layer(6 * room_width_third + 500, 3 * room_height_third + 500, "InstancesBottom", obj_fragment);

instance_create_layer(4 * room_width_third + 500, 8 * room_height_third + 500, "InstancesBottom", obj_fragment);

instance_create_layer(-8 * room_width_third + 500, -6 * room_height_third + 500, "InstancesBottom", obj_fragment);